from typing import TYPE_CHECKING, Any, Optional, Tuple

if TYPE_CHECKING:
    import uuid

from gcs_cli.commands.storage_gateway._common import make_update_command
from gcs_cli.parsing import (
    globus_command,
    hpss_options,
    storage_gateway_create_and_update_params,
    storage_gateway_id_arg,
)
from gcs_cli.sdk_client.data import HPSSStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import HPSS_CONNECTOR_ID


@globus_command("hpss", short_help="Update a HPSS Storage Gateway")
@storage_gateway_id_arg
@storage_gateway_create_and_update_params(create=False)
@hpss_options(create=False)
@make_update_command
def hpss_update(
    storage_gateway_id: "uuid.UUID",
    *,
    authentication_mech: Optional[str],
    authenticator: Optional[str],
    uda_checksum: Optional[str],
    **kwargs: Any
) -> Tuple["uuid.UUID", StorageGatewayDocument]:
    """
    Update an existing HPSS Storage Gateway.
    Any options given will replace existing values in the storage gateway's definition.
    Any options not given will retain their existing values.
    """

    policy_args = {}
    if authentication_mech:
        policy_args["authentication_mech"] = authentication_mech
    if authenticator:
        policy_args["authenticator"] = authenticator
    if uda_checksum is not None:
        policy_args["uda_checksum"] = uda_checksum

    policies = None
    if policy_args:
        policies = HPSSStoragePolicies(**policy_args)

    return (
        storage_gateway_id,
        StorageGatewayDocument(
            connector_id=HPSS_CONNECTOR_ID, policies=policies, **kwargs
        ),
    )
