from typing import (
    ClassVar,
    List,
    Type,
    TYPE_CHECKING,
)
import uuid

from .base import (
    Check,
    CheckInvoker,
    MissingPropertyCheck,
    PropertyConstraintCheck,
)

if TYPE_CHECKING:
    from ..models.acl import ACL  # noqa: F401


class ACLCheck(CheckInvoker["ACL"]):
    checkers: ClassVar[List[Type[Check["ACL"]]]] = []
    object_name = "acl"


@ACLCheck.property_checker(property_name="principal")
class AC01(MissingPropertyCheck["ACL"]):
    ...


@ACLCheck.property_checker(property_name="principal")
class AC02(PropertyConstraintCheck["ACL"]):
    constraint_name = "principal is UUID or ''"

    def value_is_ok(self, value: "ACL") -> bool:
        ok = True
        if value.principal not in [None, ""]:
            try:
                uuid.UUID(value.principal)
            except ValueError:
                ok = False
        return ok


@ACLCheck.property_checker(property_name="principal_type")
class AC03(MissingPropertyCheck["ACL"]):
    ...


@ACLCheck.property_checker(property_name="principal_type")
class AC04(PropertyConstraintCheck["ACL"]):
    constraint_name = (
        "principal_type must be identity, group, all_authenticated_users, or anonymous"
    )

    def value_is_ok(self, value: "ACL") -> bool:
        ok = True
        if value.principal_type is not None:
            if value.principal_type not in [
                "identity",
                "group",
                "all_authenticated_users",
                "anonymous",
            ]:
                ok = False
        return ok


@ACLCheck.property_checker(property_name="path")
class AC05(MissingPropertyCheck["ACL"]):
    ...


@ACLCheck.property_checker(property_name="permissions")
class AC06(MissingPropertyCheck["ACL"]):
    ...


@ACLCheck.property_checker(property_name="permissions")
class AC07(PropertyConstraintCheck["ACL"]):
    constraint_name = "permissions is r or rw"

    def value_is_ok(self, value: "ACL") -> bool:
        ok = True
        if value.permissions is not None:
            if value.permissions not in ["r", "rw"]:
                ok = False
        return ok
