from typing import List

from globus.manager.constants import ConfigurationObjectTag
import globus.manager.api.errors as errors
from .user_credential import UserCredential


class HPSSUserCredential(UserCredential):
    CONNECTOR_ID = "fb656a17-0f69-4e59-95ff-d0a62ca7bdf5"

    # Current policy data_type, used for new docs and dumping
    POLICY_DATA_TYPE = "hpss_user_credential_policies#1.0.0"
    # Supported policy data_types, current and legacy
    POLICY_DATA_TYPES = [POLICY_DATA_TYPE]

    __mapper_args__ = {
        "polymorphic_identity": (
            ConfigurationObjectTag.USER_CREDENTIAL.name + CONNECTOR_ID
        )
    }

    @property
    def connector_properties(self) -> List[str]:
        """
        Our properties, used by UserCredential to validate input.
        """
        return []

    def __init__(self, appsync_data=None, doc=None, storage_gateway=None):
        if doc:
            username = doc["username"]

            passwd_entry = storage_gateway.lookup_unix_info(username)
            if not passwd_entry:
                raise errors.InvalidUserError(username)
            display_name = passwd_entry.pw_gecos.split(",")[0] or username
            policy = doc.get("policies")
            if not policy:
                policy = {"DATA_TYPE": self.POLICY_DATA_TYPE}
            doc = {
                **doc,
                **{
                    "display_name": display_name,
                    "invalid": False,
                    "deleted": False,
                    "provisioned": True,
                    "policies": policy,
                },
            }

        super().__init__(
            appsync_data=appsync_data, doc=doc, storage_gateway=storage_gateway
        )

    def put(self, **doc):
        display_name = self.storage_gateway.get_user_display_name(self.username)
        super().put(**{**doc, **{"display_name": display_name}})

    def patch(self, **doc):
        doc.pop("display_name", None)
        super().patch(**doc)

    @property
    def connector_id(self):
        return self.CONNECTOR_ID

    @connector_id.setter
    def connector_id(self, value):
        pass

    def __repr__(self):
        return "<HPSSUserCredential {}>".format(self.id)
