import marshmallow as ma

from .semantic_version import semantic_version_expression
from .data_type import DataTypeSchema


class InfoSchema(ma.Schema):
    """
    This document contains information about the Globus Connect
    Server, including its software and supported API version
    number.
    """

    _DATA_TYPE = "info#1.0.0"
    DATA_TYPE = ma.fields.Str(
        default=_DATA_TYPE, validate=[ma.validate.Equal(_DATA_TYPE)]
    )
    manager_version = ma.fields.Str(
        validate=[ma.validate.Regexp(semantic_version_expression)]
    )
    api_version = ma.fields.Str(
        validate=[ma.validate.Regexp(semantic_version_expression)]
    )
    client_id = ma.fields.UUID()
    endpoint_id = ma.fields.UUID()
    domain_name = ma.fields.String()


info_schema = InfoSchema()

DataTypeSchema.register_schema(info_schema)
