from typing import List

import marshmallow as ma

from ..api.collection import CollectionSchema
from ..api.data_type import DataTypeSchema
from ..api.storage_gateway import StorageGatewaySchema
from ..api.user_credential import UserCredentialSchema


HPSS_CONNECTOR_ID = "fb656a17-0f69-4e59-95ff-d0a62ca7bdf5"


class HPSSStoragePoliciesSchema(ma.Schema):
    _DATA_TYPE = "hpss_storage_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )

    authentication_mech = ma.fields.String(
        validate=[ma.validate.OneOf(["krb5", "unix"])], required=True
    )
    authenticator = ma.fields.Str(
        required=True, validate=ma.validate.Regexp(r"^(auth_keyfile|auth_keytab):/")
    )
    uda_checksum = ma.fields.Boolean(required=True)

    private = ["authentication_mech", "authenticator"]
    connector_id = HPSS_CONNECTOR_ID


class HPSSCollectionPoliciesSchema(ma.Schema):
    _DATA_TYPE = "hpss_collection_policies#1.0.0"
    DATA_TYPE = ma.fields.String(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    private: List[str] = []
    connector_id = HPSS_CONNECTOR_ID


class HPSSUserCredentialPoliciesSchema(ma.Schema):
    _DATA_TYPE = "hpss_user_credential_policies#1.0.0"
    DATA_TYPE = ma.fields.String(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    private: List[str] = []
    connector_id = HPSS_CONNECTOR_ID


hpss_storage_policies_private_schema = HPSSStoragePoliciesSchema()
hpss_storage_policies_schema = HPSSStoragePoliciesSchema(
    load_only=HPSSStoragePoliciesSchema.private
)
DataTypeSchema.register_schema(hpss_storage_policies_schema)
StorageGatewaySchema.register_schema(
    public=hpss_storage_policies_schema,
    private=hpss_storage_policies_private_schema,
)
hpss_collection_policies_private_schema = HPSSCollectionPoliciesSchema()
hpss_collection_policies_schema = HPSSCollectionPoliciesSchema(
    load_only=HPSSCollectionPoliciesSchema.private
)
DataTypeSchema.register_schema(hpss_collection_policies_schema)
CollectionSchema.register_schema(
    public=hpss_collection_policies_schema,
    private=hpss_collection_policies_private_schema,
)
hpss_user_credential_policies_private_schema = HPSSUserCredentialPoliciesSchema()
hpss_user_credential_policies_schema = HPSSUserCredentialPoliciesSchema(
    load_only=HPSSUserCredentialPoliciesSchema.private
)
DataTypeSchema.register_schema(hpss_user_credential_policies_schema)
UserCredentialSchema.register_schema(
    public=hpss_user_credential_policies_schema,
    private=hpss_user_credential_policies_private_schema,
)
