from typing import List

import marshmallow as ma

from ..api.collection import CollectionSchema
from ..api.data_type import DataTypeSchema
from ..api.storage_gateway import StorageGatewaySchema
from ..api.user_credential import UserCredentialSchema
from .posix import (
    PosixStoragePoliciesSchema,
    PosixUserCredentialPoliciesSchema,
    PosixCollectionPoliciesSchema_1_1_0,
)


POSIX_STAGING_CONNECTOR_ID = "052be037-7dda-4d20-b163-3077314dc3e6"


class EnvironmentSchema(ma.Schema):
    name = ma.fields.String(required=True)
    value = ma.fields.String(required=True)


class PosixStagingStoragePoliciesSchema(PosixStoragePoliciesSchema):
    _DATA_TYPE = "posix_staging_storage_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    stage_app = ma.fields.String(required=True)
    environment = ma.fields.List(ma.fields.Nested(EnvironmentSchema))

    private = PosixStoragePoliciesSchema.private + ["stage_app", "environment"]
    connector_id = POSIX_STAGING_CONNECTOR_ID


class PosixStagingCollectionPoliciesSchema(PosixCollectionPoliciesSchema_1_1_0):
    _DATA_TYPE = "posix_staging_collection_policies#1.0.0"
    DATA_TYPE = ma.fields.String(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    private = PosixCollectionPoliciesSchema_1_1_0.private + []  # type: List[str]
    connector_id = POSIX_STAGING_CONNECTOR_ID


class PosixStagingUserCredentialPoliciesSchema(PosixUserCredentialPoliciesSchema):
    _DATA_TYPE = "posix_staging_user_credential_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    connector_id = POSIX_STAGING_CONNECTOR_ID
    private = PosixUserCredentialPoliciesSchema.private + []  # type: List[str]


posix_staging_storage_policies_schema = PosixStagingStoragePoliciesSchema(
    load_only=PosixStagingStoragePoliciesSchema.private
)
posix_staging_storage_policies_private_schema = PosixStagingStoragePoliciesSchema()
posix_staging_user_credential_policies_schema = (
    PosixStagingUserCredentialPoliciesSchema()
)
posix_staging_collection_policies_schema = PosixStagingCollectionPoliciesSchema(
    load_only=PosixStagingCollectionPoliciesSchema.private,
)
posix_staging_collection_policies_private_schema = (
    PosixStagingCollectionPoliciesSchema()
)

DataTypeSchema.register_schema(posix_staging_storage_policies_schema)
DataTypeSchema.register_schema(posix_staging_user_credential_policies_schema)
DataTypeSchema.register_schema(posix_staging_collection_policies_schema)

StorageGatewaySchema.register_schema(
    public=posix_staging_storage_policies_schema,
    private=posix_staging_storage_policies_private_schema,
)

UserCredentialSchema.register_schema(
    public=posix_staging_user_credential_policies_schema
)

CollectionSchema.register_schema(
    public=posix_staging_collection_policies_schema,
    private=posix_staging_collection_policies_private_schema,
)
