import enum
import marshmallow as ma


class EnumField(ma.fields.Field):
    def __init__(self, *args, enum_class=enum.Enum, **kwargs):
        if not issubclass(enum_class, enum.Enum):
            raise ValueError("{} is not an enum".format(enum_class.__name__))
        self.enum_class = enum_class
        super().__init__(*args, **kwargs)

    def _serialize(self, value, attr, obj, **kwargs):
        if isinstance(value, self.enum_class):
            return value.name
        return value

    def _deserialize(self, value, attr, data, **kwargs):
        try:
            return getattr(self.enum_class, value)
        except Exception as e:
            raise ma.ValidationError(
                "{} not a value for {}".format(value, self.enum_class.__name__), e
            )
