"""
globus-connect-server endpoint setup
"""
import uuid

import click
from gcs_cli.parsing.command_decorator import globus_command

from gcs_cli.parsing.types import CommaDelimitedList, StringOrNull


@globus_command(
    "setup",
    short_help="Setup a Globus Connect Server endpoint",
    help="Setup a Globus Connect Server endpoint",
    disable_options=["format", "explicit_host"],
)
@click.argument("display_name")
@click.option(
    "--public/--private",
    "public",
    default=True,
    help="Set the endpoint to be public or private",
    show_default=True,
)
@click.option(
    "--description",
    type=StringOrNull(),
    help="Description of the endpoint",
)
@click.option(
    "--info-link",
    type=StringOrNull(),
    help="Link to a web page with more information about the endpoint",
)
@click.option(
    "--contact-info",
    type=StringOrNull(),
    help="Other non-email contact information for the endpoint",
)
@click.option(
    "--contact-email",
    type=StringOrNull(),
    help="Email address of the support contact for this endpoint",
    required=True,
)
@click.option(
    "--organization",
    type=StringOrNull(),
    help="Organization that owns this endpoint",
    required=True,
)
@click.option(
    "--department",
    type=StringOrNull(),
    help="Department within the organization that runs the endpoint",
)
@click.option(
    "--keywords",
    type=CommaDelimitedList(),
    help="Comma separated list of keywords to help searches for the endpoint",
)
@click.option(
    "--client-id",
    "-c",
    help="Auth Client ID for this Globus Connect Server Endpoint",
    type=click.types.UUIDParameterType(),
    required=True,
)
@click.option(  # type: ignore # hidden/required
    "--secret",
    "-s",
    help="Secret for the auth client",
    required=True,
    hidden=True,
    prompt=True,
    hide_input=True,
)
@click.option(
    "--deployment-key",
    "-d",
    help="Path for deployment key configuration.",
    type=click.Path(exists=False),
    default="deployment-key.json",
)
@click.option(
    "--owner",
    "-o",
    help="Globus username of the owner of this endpoint. This user will be "
    "granted the owner role for this endpoint.",
    required=True,
)
@click.option(
    "--agree-to-letsencrypt-tos",
    help="Agree to the Let's Encrypt Terms of Service: "
    "https://letsencrypt.org/repository/",
    is_flag=True,
)
@click.option(
    "--dont-set-advertised-owner",
    help="Skip the interactive step to set the endpoint advertised owner",
    is_flag=True,
)
def endpoint_setup(
    display_name: str,
    client_id: uuid.UUID,
    secret: str,
    deployment_key: str,
    owner: str,
    agree_to_letsencrypt_tos: bool,
    dont_set_advertised_owner: bool,
    **kwargs
) -> None:
    from globus.manager.scripts.endpoint_setup import endpoint_setup as es

    es(
        display_name,
        client_id,
        secret,
        deployment_key,
        owner,
        agree_to_letsencrypt_tos,
        dont_set_advertised_owner,
        **kwargs
    )
