"""
Module to configure an apache server on a debian >= 8 or ubuntu >= 14.04
system.
"""
from __future__ import print_function, absolute_import

from subprocess import Popen
import os

__apache_mods_enabled_file = "/etc/apache2/mods-enabled/mod_{name}.load"
__apache_sites_available_file = "/etc/apache2/sites-enabled/{name}.conf"
__apache_sites_enabled_file = "/etc/apache2/sites-enabled/{name}.conf"
__apache_conf_enabled_file = "/etc/apache2/conf-enabled/{name}.conf"

__gcs_enabled_mod = "/var/lib/globus-connect-server/enabled_mod_{name}"
__gcs_created_site = "/var/lib/globus-connect-server/created_site_{name}"
__gcs_enabled_site = "/var/lib/globus-connect-server/enabled_site_{name}"
__gcs_enabled_conf = "/var/lib/globus-connect-server/enabled_conf_{name}"


service = "apache2"
required_modules = ["headers", "proxy", "proxy_http", "rewrite", "ssl"]
required_confs = ["tls-mod-globus"]


def site_add(name, site_config, force=False, enable=False):
    site_available = __apache_sites_available_file.format(name=name)
    created_site_path = __gcs_created_site.format(name=name)
    created_new_config = False

    if not os.path.exists(site_available):
        created_new_config = True

    with open(site_available, "w") as f:
        f.write(site_config)

    if created_new_config:
        open(created_site_path, "w").close()

    if enable:
        site_enable(name)


def site_remove(name):
    site_available = __apache_sites_available_file.format(name=name)
    created_site_path = __gcs_created_site.format(name=name)

    site_disable(name)

    if os.path.exists(created_site_path):
        try:
            os.remove(site_available)
        except FileNotFoundError:
            pass
        try:
            os.remove(created_site_path)
        except FileNotFoundError:
            pass


def module_enable(module, force=False):
    mod_enabled_path = __apache_mods_enabled_file.format(name=module)
    gcs_enabled_mod = __gcs_enabled_mod.format(name=module)

    if force or not os.path.exists(mod_enabled_path):
        enabler = Popen(["/usr/sbin/a2enmod", module], stdin=None)
        enabler.communicate()
        if enabler.returncode == 0:
            open(gcs_enabled_mod, "w").close()


def module_disable(module):
    gcs_enabled_mod = __gcs_enabled_mod.format(name=module)
    if os.path.exists(gcs_enabled_mod):
        disabler = Popen(["/usr/sbin/a2dismod", module], stdin=None)
        disabler.communicate()
        if disabler.returncode == 0:
            try:
                os.remove(gcs_enabled_mod)
            except FileNotFoundError:
                pass


def site_enable(name, force=False):
    enabled_site_path = __apache_sites_enabled_file.format(name=name)
    gcs_enabled_site_path = __gcs_enabled_site.format(name=name)

    if force or not os.path.exists(enabled_site_path):
        enabler = Popen(["/usr/sbin/a2ensite", name], stdin=None)
        enabler.communicate()
        if enabler.returncode == 0:
            open(gcs_enabled_site_path, "w").close()


def site_disable(name):
    gcs_enabled_site = __gcs_enabled_site.format(name=name)

    if os.path.exists(gcs_enabled_site):
        disabler = Popen(["/usr/sbin/a2dissite", name], stdin=None)
        disabler.communicate()
        if disabler.returncode == 0:
            site_remove(gcs_enabled_site)


def conf_enable(name, force=False):
    enabled_conf_path = __apache_conf_enabled_file.format(name=name)
    gcs_enabled_conf_path = __gcs_enabled_conf.format(name=name)

    if force or not os.path.exists(enabled_conf_path):
        enabler = Popen(["/usr/sbin/a2enconf", name], stdin=None)
        enabler.communicate()
        if enabler.returncode == 0:
            open(gcs_enabled_conf_path, "w").close()


def conf_disable(name):
    gcs_enabled_conf = __gcs_enabled_conf.format(name=name)

    if os.path.exists(gcs_enabled_conf):
        disabler = Popen(["/usr/sbin/a2disconf", name], stdin=None)
        disabler.communicate()
        if disabler.returncode == 0:
            try:
                os.remove(gcs_enabled_conf)
            except FileNotFoundError:
                pass
