"""joined_table_inheritance

Revision ID: 7135f0be454a
Revises: c39c7d4591a9
Create Date: 2019-09-26 09:23:11.100205

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_json

# revision identifiers, used by Alembic.
revision = '7135f0be454a'
down_revision = 'c39c7d4591a9'
branch_labels = None
depends_on = None


def upgrade():
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")
    connection = op.get_bind()
    # Add a storage_gateway_id field to  collections table
    op.rename_table('collection', 'collection_old')
    op.create_table('collection',
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('path', sa.TEXT(), nullable=False),
        sa.Column('domain', sa.TEXT(), nullable=False),
        sa.Column(
            'collection_policies',
            sqlalchemy_json.NestedMutableJson(),
            nullable=True),
        sa.Column('user_credential_id', sa.TEXT(), nullable=True),
        sa.Column('storage_gateway_id', sa.TEXT(), nullable=False),
        sa.ForeignKeyConstraint(
            ['id'],
            ['configuration_object.id'],
            name='col_id'),
        sa.ForeignKeyConstraint(
            ['storage_gateway_id'],
            ['storage_gateway.id'], ),
        sa.ForeignKeyConstraint(
            ['user_credential_id'],
            ['user_credential.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    # Remove links to unauthenticated S3 and mapped collection dummy credentials
    op.execute(
        r"""
        INSERT INTO collection (
            id,
            path,
            domain,
            collection_policies,
            user_credential_id,
            storage_gateway_id)
        SELECT
            col.id,
            col.path,
            col.domain,
            col.collection_policies,
            col.user_credential_id,
            CASE
                WHEN uc.storage_gateway_user_id
                    IN ('UNAUTHENTICATED', ':' || 'globus-mapping' || ':')
                THEN
                    NULL
                ELSE
                    uc.storage_gateway_user_id
            END
        FROM collection_old as col
        JOIN user_credential as uc on col.user_credential_id = uc.id
        """)
    # Drop collections_old table
    op.drop_table('collection_old')

    # Remove unauthenticated S3 and mapped collection dummy credentials
    op.execute(
        """
        DELETE FROM user_credential
        WHERE
        storage_gateway_user_id IN ('UNAUTHENTICATED', ':' || 'globus-mapping' || ':')
        """)

    # Rename role table
    op.rename_table('role', 'role_old')
    # Recreate role table without created_at, updated_at (those are in
    # configuration_object table), and adding a foreign key constraint linking
    # id to configuration_object.id
    op.create_table('role',
        sa.Column(
            'id',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'principal_type',
            sa.Enum('identity',
                'group',
                name='principaltype'),
            nullable=False),
        sa.Column(
            'identity_id',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'role',
            sa.Enum(
                'endpoint_administrator',
                'endpoint_activity_manager',
                'endpoint_activity_monitor',
                'collection_administrator',
                'collection_activity_manager',
                'collection_activity_monitor',
                'collection_access_manager',
                name='roletype'),
            nullable=False),
        sa.Column(
            'resource',
            sa.TEXT(),
            nullable=True),
        sa.ForeignKeyConstraint(
            ['id'],
            ['configuration_object.id'],
        ),
        sa.ForeignKeyConstraint(
            ['resource'],
            ['collection.id']),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint(
            'principal_type',
            'identity_id',
            'role',
            'resource',
            name='unique_collection_role')
    )
    # Add old roles into new table
    op.execute(
        """
        INSERT INTO role (
            id,
            principal_type,
            identity_id,
            role,
            resource)
        SELECT id, principal_type, identity_id, role, resource
        FROM role_old
        """)
    # drop old table
    op.drop_table("role_old")
    # back up storage_gateway table
    op.rename_table('storage_gateway', 'storage_gateway_old')
    # Recreate storage_gateway table without created_at, updated_at (those
    # are in configuration_object table), and adding a foreign key
    # constraint linking id to configuration_object.id
    op.create_table(
        'storage_gateway',
        sa.Column('id', sa.TEXT(),
            nullable=False),
        sa.Column(
            'display_name',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'credential',
            sqlalchemy_json.NestedMutableJson(),
            nullable=True),
        sa.Column(
            'storage_policies',
            sqlalchemy_json.NestedMutableJson(),
            nullable=True),
        sa.Column(
            'allow_guest_collections',
            sa.BOOLEAN(),
            server_default='0',
            nullable=False),
        sa.Column(
            'allow_mapped_collections',
            sa.BOOLEAN(),
            server_default='0',
            nullable=False),
        sa.Column(
            'require_high_assurance',
            sa.BOOLEAN(),
            server_default='0',
            nullable=False),
        sa.Column(
            'deleted',
            sa.Boolean(),
            server_default='0',
            nullable=False),
        sa.Column(
            'connector_id',
            sa.TEXT(),
            nullable=False),
        sa.ForeignKeyConstraint(
            ['connector_id'],
            ['connector.id'], ),
        sa.ForeignKeyConstraint(
            ['id'],
            ['configuration_object.id'],),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('display_name')
    )
    # Copy the data from the old to the new table
    op.execute(
        """
        INSERT INTO storage_gateway
        (id, display_name, credential, storage_policies,
         allow_guest_collections, allow_mapped_collections,
         require_high_assurance, deleted, connector_id)
         SELECT id, display_name, credential, storage_policies,
         allow_guest_collections, allow_mapped_collections,
         require_high_assurance, deleted, connector_id
         FROM storage_gateway_old
        """)
    # Remove the backup
    op.drop_table('storage_gateway_old')

    op.rename_table('user_credential', 'user_credential_old')
    op.create_table(
        'user_credential',
        sa.Column('id',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'display_name',
            sa.TEXT(),
            nullable=True),
        sa.Column(
            'identity_id',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'storage_credential',
            sa.TEXT(),
            nullable=True),
        sa.Column(
            'storage_gateway_user_id',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'deleted',
            sa.Boolean(),
            server_default='0',
            nullable=False),
        sa.Column(
            'invalid',
            sa.Boolean(),
            server_default='0',
            nullable=False),
        sa.Column(
            'storage_gateway_id',
            sa.TEXT(),
            nullable=False),
        sa.ForeignKeyConstraint(
            ['id'],
            ['configuration_object.id'], ),
        sa.ForeignKeyConstraint(
            ['storage_gateway_id'],
            ['storage_gateway.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint(
            'storage_gateway_id',
            'storage_gateway_user_id',
            'identity_id')
    )
    op.execute(
        """
        INSERT INTO user_credential (
            id, display_name, identity_id,
            storage_credential, storage_gateway_user_id,
            deleted, invalid, storage_gateway_id)
        SELECT id, display_name, identity_id,
            storage_credential, storage_gateway_user_id,
            deleted, invalid, storage_gateway_id
        FROM user_credential_old
        """)
    op.drop_table("user_credential_old")


def downgrade():
    raise RuntimeError("downgrade migration not implemented")
