"""Add sharing policies table

Revision ID: d0eed3bd1fa1
Revises: b8c8667d06e3
Create Date: 2021-02-10 19:09:18.088127

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'd0eed3bd1fa1'
down_revision = 'b8c8667d06e3'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('sharing_policy',
    sa.Column('id', sa.TEXT(), nullable=False),
    sa.Column('collection_id', sa.TEXT(), nullable=True),
    sa.ForeignKeyConstraint(['collection_id'], ['collection.id'], ),
    sa.ForeignKeyConstraint(['id'], ['configuration_object.id'], name='col_id'),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###
    # Remove DOMAIN from ConfigurationObjectTag and add SHARING_POLICY
    op.execute("PRAGMA foreign_keys = OFF")
    op.execute("PRAGMA legacy_alter_table=ON")
    op.rename_table("configuration_object", "configuration_object_old")
    op.create_table(
        "configuration_object",
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.Column("discriminator", sa.TEXT(), nullable=False),
        sa.Column(
            "tag",
            sa.Enum(
                "ACME_ACCOUNT",
                "CERTIFICATE",
                "COLLECTION",
                "ENDPOINT",
                "KEY_CHAIN",
                "ROLE",
                "SHARING_POLICY",
                "STORAGE_GATEWAY",
                "USER_CREDENTIAL",
                "WILDCARD_DOMAIN",
                name="configurationobjecttag",
            ),
            nullable=False,
        ),
        sa.Column("id", sa.TEXT(), nullable=False),
        sa.Column("serial", sa.INTEGER(), nullable=False),
        sa.Column("configuration", sa.TEXT(), nullable=False),
        sa.Column("deleted", sa.BOOLEAN(), server_default="0", nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.execute(
        """
        INSERT INTO configuration_object
        (id, created_at, updated_at, discriminator, tag,
         serial, configuration, deleted)
        SELECT id, created_at, updated_at, discriminator, tag,
         serial, configuration,
         case when deleted is null then 0 else 1 end
         from configuration_object_old
        """
    )
    op.drop_table("configuration_object_old")

def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('sharing_policy')
    # ### end Alembic commands ###
    # Add DOMAIN to ConfigurationObjectTag and remove SHARING_POLICY
    op.execute("PRAGMA foreign_keys = OFF")
    op.execute("PRAGMA legacy_alter_table=ON")
    op.rename_table("configuration_object", "configuration_object_old")
    op.create_table(
        "configuration_object",
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("updated_at", sa.DateTime(), nullable=False),
        sa.Column("discriminator", sa.TEXT(), nullable=False),
        sa.Column(
            "tag",
            sa.Enum(
                "ACME_ACCOUNT",
                "CERTIFICATE",
                "COLLECTION",
                "DOMAIN",
                "ENDPOINT",
                "KEY_CHAIN",
                "ROLE",
                "STORAGE_GATEWAY",
                "USER_CREDENTIAL",
                "WILDCARD_DOMAIN",
                name="configurationobjecttag",
            ),
            nullable=False,
        ),
        sa.Column("id", sa.TEXT(), nullable=False),
        sa.Column("serial", sa.INTEGER(), nullable=False),
        sa.Column("configuration", sa.TEXT(), nullable=False),
        sa.Column("deleted", sa.BOOLEAN(), server_default="0", nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.execute(
        """
        INSERT INTO configuration_object
        (id, created_at, updated_at, discriminator, tag,
         serial, configuration, deleted)
        SELECT id, created_at, updated_at, discriminator, tag,
         serial, configuration,
         case when deleted is null then 0 else 1 end
         from configuration_object_old
        """
    )
    op.drop_table("configuration_object_old")
